@@ -2,7 +2,7 @@
    daemon.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,8 +33,6 @@
 #endif /* USE_LIBWRAP */
 
 static RETSIGTYPE sig_cld(int);
-static void service_unavailable(int);
-static void set_unprivileged_user(void);
 
 static int nchild;
 
@@ -104,8 +102,8 @@ sig_cld(int code)
  then close a transmission channel.
 *************************************/
 
-static void
-service_unavailable(int sd_client)
+void
+service_unavailable(NET_STREAM *sd_client)
 {
 	char buf[LINEBUFFER+1];
 
@@ -113,8 +111,9 @@ service_unavailable(int sd_client)
 	"421 %s Service not available, closing transmission channel."CRLF,
 		(topt & T_LOCAL_MTA) ? "localhost" : session.mta);
 
-	swrite(SERVER, (void *)sd_client, buf);
-	close_socket(sd_client);
+	swrite(SERVER, *sd_client, buf);
+	stream_close(*sd_client);
+	stream_destroy(sd_client);
 	return;
 }
 
@@ -123,7 +122,7 @@ service_unavailable(int sd_client)
  (if possible).
 **************************/
 
-static void
+void
 set_unprivileged_user(void)
 {
 	if (topt & T_USER_NOTPRIVIL) {
@@ -140,6 +139,30 @@ set_unprivileged_user(void)
 	return;
 }
 
+int
+anubis_child_main (NET_STREAM *sd_client, struct sockaddr_in *addr)
+{
+	int rc;
+
+	topt &= ~T_FOREGROUND;
+	
+#ifdef WITH_GSASL
+	switch (anubis_mode) {
+	case anubis_transparent:
+		rc = anubis_transparent_mode(sd_client, addr);
+		break;
+
+	case anubis_authenticate:
+		rc = anubis_authenticate_mode(sd_client, addr);
+	}
+#else
+	rc = anubis_transparent_mode(sd_client, addr);
+#endif /* WITH_GSASL */
+
+	net_close_stream(sd_client);
+	return rc;
+}
+
 /**************
   DAEMON loop
 ***************/
@@ -149,8 +172,6 @@ loop(int sd_bind)
 {
 	struct sockaddr_in addr;
 	pid_t childpid = 0;
-	int sd_client = 0;
-	int sd_server = 0;
 	socklen_t addrlen;
 #ifdef USE_LIBWRAP
 	struct request_info req;
@@ -164,29 +185,35 @@ loop(int sd_bind)
 
 	info(VERBOSE, _("GNU Anubis is running..."));
 
-	for (;;)
-	{
-		sd_client = accept(sd_bind, (struct sockaddr *)&addr, &addrlen);
-		if (sd_client < 0) {
+	for (;;) {
+		NET_STREAM sd_client = NULL;
+		int fd = accept(sd_bind, (struct sockaddr *)&addr, &addrlen);
+		if (fd < 0) {
 			if (errno == EINTR)
 				continue;
 			else {
-				anubis_error(SOFT, _("accept() failed: %s."), strerror(errno));
+				anubis_error(SOFT,
+					     _("accept() failed: %s."),
+					     strerror(errno));
 				continue;
 			}
 		}
 
+		/* Create the TCP stream */
+		net_create_stream(&sd_client, fd);
+		
 		/*
 		   Check the TCP wrappers settings.
 		*/
 
 #ifdef USE_LIBWRAP
-		request_init(&req, RQ_DAEMON, "anubis", RQ_FILE, sd_client, 0);
+		request_init(&req, RQ_DAEMON, "anubis", RQ_FILE, fd, 0);
 		fromhost(&req);
 		if (hosts_access(&req) == 0) {
-			info(NORMAL, _("TCP wrappers: connection from %s:%u rejected."),
-				inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
-			service_unavailable(sd_client);
+			info(NORMAL,
+			     _("TCP wrappers: connection from %s:%u rejected."),
+			     inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
+			service_unavailable(&sd_client);
 			continue;
 		}
 #endif /* USE_LIBWRAP */
@@ -204,7 +231,7 @@ loop(int sd_bind)
 		if (nchild > MAXCLIENTS) {
 			info(NORMAL, _("Too many clients. Connection from %s:%u rejected."),
 				inet_ntoa(addr.sin_addr), ntohs(addr.sin_port));
-			service_unavailable(sd_client);
+			service_unavailable(&sd_client);
 			nchild--;
 		}
 		else {
@@ -213,145 +240,16 @@ loop(int sd_bind)
 
 			childpid = fork();
 			if (childpid == -1)
-				anubis_error(HARD, _("daemon: Can't fork. %s."), strerror(errno));
+				anubis_error(HARD,
+					     _("daemon: Can't fork. %s."),
+					     strerror(errno));
 			else if (childpid == 0) { /* a child process */
-				int rs = 0;
-				int cs = 0;
+				/* FIXME */
 				signal(SIGCHLD, SIG_IGN);
-
-				rs = auth_ident(&addr,
-						session.client,
-						sizeof(session.client));
-
-				if ((topt & T_DROP_UNKNOWN_USER) && !rs) {
-				  service_unavailable(sd_client);
-				  quit(0);
-				}
-
-				parse_transmap(&cs,
-					       rs ? session.client : 0,
-					       inet_ntoa(addr.sin_addr),
-					       session.client,
-					       sizeof(session.client));
-				
-				if (cs == 1) {
-					anubis_changeowner(session.client);
-					auth_tunnel();
-				}
-				else if (rs
-					 && cs == -1
-					 && ntohl(addr.sin_addr.s_addr)
-					 == INADDR_LOOPBACK) {
-					if (check_username(session.client)) {
-						anubis_changeowner(session.client);
-						auth_tunnel();
-					}
-					else
-						set_unprivileged_user();
-				}
-				else
-					set_unprivileged_user();
-
-				if (!(topt & T_LOCAL_MTA)
-				    && strlen(session.mta) == 0) {
-					anubis_error(HARD, _("The MTA has not been specified. "
-						"Set the REMOTE-MTA or LOCAL-MTA."));
-					close_socket(sd_client);
-					quit(EXIT_FAILURE);
-				}
-
-				/*
-				   Protection against a loop connection.
-				*/
-
-				if (!(topt & T_LOCAL_MTA)) {
-					unsigned long inaddr;
-					struct sockaddr_in ad;
-
-					memset(&ad, 0, sizeof(ad));
-					inaddr = inet_addr(session.mta);
-					if (inaddr != INADDR_NONE)
-						memcpy(&ad.sin_addr, &inaddr, sizeof(inaddr));
-					else {
-						struct hostent *hp = 0;
-						hp = gethostbyname(session.mta);
-						if (hp == 0) {
-							hostname_error(session.mta);
-							close_socket(sd_client);
-							quit(EXIT_FAILURE);
-						}
-						else {
-							if (hp->h_length != 4 && hp->h_length != 8) {
-								anubis_error(HARD,
-								_("Illegal address length received for host %s"), session.mta);
-								close_socket(sd_client);
-								quit(EXIT_FAILURE);
-							}
-							else {
-								memcpy((char *)&ad.sin_addr.s_addr, hp->h_addr,
-									hp->h_length);
-							}
-						}
-					}
-					if (ntohl(ad.sin_addr.s_addr) == INADDR_LOOPBACK
-					&& session.tunnel_port == session.mta_port) {
-						anubis_error(SOFT, _("Loop not allowed. Connection rejected."));
-						close_socket(sd_client);
-						quit(EXIT_FAILURE);
-					}
-				}
-
-				alarm(300);
-				if (topt & T_LOCAL_MTA) {
-					sd_server = make_local_connection(session.execpath, session.execargs);
-					if (sd_server == -1) {
-						service_unavailable(sd_client);
-						quit(EXIT_FAILURE);
-					}
-				}
-				else {
-					sd_server = make_remote_connection(session.mta, session.mta_port);
-					if (sd_server == -1)
-						service_unavailable(sd_client);
-				}
-				alarm(0);
-
-				if (!(topt & T_ERROR)) {
-					remote_client = (void *)sd_client;
-					remote_server = (void *)sd_server;
-					alarm(900);
-					smtp_session();
-					alarm(0);
-
-#ifdef USE_SSL
-					net_close(CLIENT, secure.client);
-					net_close(SERVER, secure.server);
-					secure.server = 0;
-					secure.client = 0;
-#endif
-				}
-				close_socket(sd_server);
-				close_socket(sd_client);
-
-				if (topt & T_ERROR)
-					info(NORMAL, _("Connection terminated."));
-				else
-					info(NORMAL, _("Connection closed successfully."));
-	
-#ifdef HAVE_PAM	
-				pam_retval = pam_close_session(pamh, 0);
-				if (pam_retval == PAM_SUCCESS)
-					info(VERBOSE, _("PAM: Session closed."));
-				if (pam_end(pamh, pam_retval) != PAM_SUCCESS) {
-					pamh = NULL;
-					info(NORMAL, _("PAM: failed to release authenticator."));
-					quit(EXIT_FAILURE);
-				}
-#endif /* HAVE_PAM */
-
-				quit(0);
+				quit(anubis_child_main(&sd_client, &addr));
 			}
-			close_socket(sd_client);
+			
+			net_close_stream(&sd_client);
 		}
 		topt &= ~T_ERROR;
 		cleanup_children();
@@ -402,7 +300,7 @@ _stdio_read(void *sd, char *data, size_t size, size_t *nbytes)
 }
 
 static const char *
-_stdio_strerror(int rc)
+_stdio_strerror(void *ignored_data, int rc)
 {
 	return strerror(rc);
 }
@@ -410,15 +308,15 @@ _stdio_strerror(int rc)
 void
 stdinout(void)
 {
-	int sd_client = 0; 
-	int sd_server = 0;
+	NET_STREAM sd_client = NULL; 
+	NET_STREAM sd_server = NULL;
 
 	topt &= ~T_SSL;
 	topt |= T_FOREGROUND;
 	topt |= T_SMTP_ERROR_CODES;
 	
-	anubis_getlogin(session.client, sizeof(session.client));
-	auth_tunnel(); /* session.client = session.supervisor */
+	anubis_getlogin(session.clientname, sizeof(session.clientname));
+	auth_tunnel(); /* session.clientname = session.supervisor */
 
 	if (!(topt & T_LOCAL_MTA) && (strlen(session.mta) == 0)) {
 		options.termlevel = NORMAL;
@@ -428,26 +326,36 @@ stdinout(void)
 		return;
 	}
 
+	net_create_stream(&sd_client, 0);
+	stream_set_read(sd_client, _stdio_read);
+	stream_set_write(sd_client, _stdio_write);
+	stream_set_strerror(sd_client, _stdio_strerror);
+
 	alarm(300);
 	if (topt & T_LOCAL_MTA)
-		sd_server = make_local_connection(session.execpath, session.execargs);
+		sd_server = make_local_connection(session.execpath,
+						  session.execargs);
 	else
-		sd_server = make_remote_connection(session.mta, session.mta_port);
+		sd_server = make_remote_connection(session.mta,
+						   session.mta_port);
 	alarm(0);
 
-	if (sd_server == -1) {
-		service_unavailable(sd_client);
+	if (sd_server == NULL) {
+		service_unavailable(&sd_client);
 		free_mem();
 		return;
 	}
-	remote_client = (void *)sd_client;
-	remote_server = (void *)sd_server;
-	net_set_io(CLIENT, _stdio_read, _stdio_write, NULL, _stdio_strerror);
-	net_set_io(SERVER, _stdio_read, _stdio_write, NULL, _stdio_strerror);
-	smtp_session();
+	stream_set_read(sd_server, _stdio_read);
+	stream_set_write(sd_server, _stdio_write);
+	stream_set_strerror(sd_server, _stdio_strerror);
+
+	remote_client = sd_client;
+	remote_server = sd_server;
+
+	smtp_session_transparent();
 	cleanup_children();
 	
-	close_socket(sd_server);
+	net_close_stream(&sd_server);
 	free_mem();
 	return;
 }
