@@ -2,7 +2,7 @@
    errs.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ anubis_error(int method, const char *fmt, ...)
 		va_end(arglist);
 #ifdef HAVE_SYSLOG
 		if ((topt & T_DAEMON) && !(topt & T_FOREGROUND)) {
-			syslog(LOG_ERR | LOG_MAIL, msg);
+			syslog(LOG_ERR | LOG_MAIL, "%s", msg);
 			if (options.ulogfile && options.uloglevel >= FAILS)
 				filelog(options.ulogfile, msg);
 		}
@@ -83,12 +83,6 @@ socket_error(const char *msg)
 		     msg ? msg : strerror(errno));
 }
 
-void
-socks_error(const char *msg)
-{
-	anubis_error(HARD, _("SOCKS proxy: %s"), msg);
-}
-
 void
 hostname_error(char *host)
 {
