@@ -2,7 +2,7 @@
    extern.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,52 +25,55 @@
 extern const char version[];
 extern const char copyright[];
 
+struct assoc {
+ char *key;
+ char *value;
+};
+
 struct options_struct {
  int termlevel;
  int uloglevel;
  char *ulogfile;
  char *tracefile;
 #ifdef WITH_GUILE
- char *guile_logfile;
-#endif /* WITH_GUILE */
+ char *glogfile;
+#endif
  char *altrc;
 };
 
 struct session_struct {
- char tunnel[65];
+ char anubis[65];
  char mta[65];
  char mta_username[65];
  char mta_password[65];
+ char supervisor[65];
+ char clientname[65];
+ char notprivileged[65];
+ char *rcfile_name;
+ char *execpath;
+ char **execargs;
+ unsigned int anubis_port;
+ unsigned int mta_port;
+#ifdef USE_SOCKS_PROXY
  char socks[65];
  char socks_username[65];
  char socks_password[65];
- char client[65];
- char supervisor[65];
- char notprivileged[65];
- char *execpath;
- char **execargs;
- unsigned int tunnel_port;
- unsigned int mta_port;
  unsigned int socks_port;
-};
-
-struct assoc {
- char *key;
- char *value;
+#endif /* USE_SOCKS_PROXY */
 };
 
 struct message_struct {
- LIST *commands; /* Associative list of SMTP commands */
- LIST *header;   /* Associative list of RFC822 headers */
- LIST *mime_hdr; /* List of lines before the first boundary marker */
- char *body;            /* Message body */
- char *boundary;        /* Additional data */
+ ANUBIS_LIST *commands;       /* Associative list of SMTP commands */
+ ANUBIS_LIST *header;         /* Associative list of RFC822 headers */
+ ANUBIS_LIST *mime_hdr;       /* List of lines before the first boundary marker */
+ char *body;           /* Message body */
+ char *boundary;       /* Additional data */
 };
 
 #ifdef USE_SSL
 struct secure_struct {
- void *client;
- void *server;
+ NET_STREAM client;
+ NET_STREAM server;
  char *cafile;
  char *cert;
  char *key;
@@ -82,13 +85,17 @@ extern struct options_struct options;
 extern struct session_struct session;
 
 extern unsigned long topt;
-extern void *remote_client;
-extern void *remote_server;
+extern NET_STREAM remote_client;
+extern NET_STREAM remote_server;
+
+extern char *anubis_domain;
 
 #ifdef HAVE_PAM
  extern pam_handle_t *pamh;
  extern int pam_retval;
 #endif /* HAVE_PAM */
 
+extern ANUBIS_MODE anubis_mode;
+
 /* EOF */
 
