@@ -2,7 +2,7 @@
    guile.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003 The Anubis Team.
+   Copyright (C) 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -67,17 +67,17 @@ guile_ports_open(void)
 {
 	SCM port;
 	int fd = -1;
-	char *name = options.guile_logfile;
+	char *name = options.glogfile;
 
 	if (topt & (T_FOREGROUND_INIT|T_STDINOUT))
 		return;
 	
 	if (name) {
-		fd = open(options.guile_logfile, O_CREAT|O_WRONLY|O_APPEND, 0644);
+		fd = open(options.glogfile, O_CREAT|O_WRONLY|O_APPEND, 0644);
 		if (fd == -1) {
 			anubis_error(SOFT,
 				     _("cannot open guile output file %s: %s"),
-				     options.guile_logfile, strerror(errno));
+				     options.glogfile, strerror(errno));
 		}
 	} else
 		name = "/dev/null";
@@ -141,10 +141,10 @@ eval_catch_handler(void *data, SCM tag, SCM throw_args)
 }
 
 
-static LIST *
+static ANUBIS_LIST *
 guile_to_anubis(SCM cell)
 {
-	static LIST *list;
+	static ANUBIS_LIST *list;
 
 	list = list_create();
 	for (; cell != SCM_EOL; cell = SCM_CDR(cell)) {
@@ -161,7 +161,7 @@ guile_to_anubis(SCM cell)
 }
 
 static SCM
-anubis_to_guile(LIST *list)
+anubis_to_guile(ANUBIS_LIST *list)
 {
 	ASSOC *asc;
 	ITERATOR *itr;
@@ -196,7 +196,7 @@ anubis_to_guile(LIST *list)
 }
 
 static SCM
-list_to_args(LIST *arglist)
+list_to_args(ANUBIS_LIST *arglist)
 {
 	char *p;
 	ITERATOR *itr;
@@ -244,7 +244,7 @@ list_to_args(LIST *arglist)
 /* (define (postproc header-list body) */
 
 void
-guile_process_proc(LIST *arglist, MESSAGE *msg)
+guile_process_proc(ANUBIS_LIST *arglist, MESSAGE *msg)
 {
 	char *procname;
 	SCM arg_hdr, arg_body;
@@ -359,14 +359,14 @@ static struct rc_kwdef guile_rule_kw[] = {
 };
 
 int
-guile_parser(int method, int key, LIST *arglist,
+guile_parser(int method, int key, ANUBIS_LIST *arglist,
 	     void *inv_data, void *func_data, MESSAGE *msg)
 {
 	char *arg = list_item(arglist, 0);
 	switch (key) {
 	case KW_GUILE_OUTPUT:
-		xfree(options.guile_logfile);
-		options.guile_logfile = strdup(arg);
+		xfree(options.glogfile);
+		options.glogfile = strdup(arg);
 		break;
 		
 	case KW_GUILE_DEBUG:
