@@ -2,7 +2,7 @@
    headers.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -105,6 +105,7 @@
 
 #if defined(USE_GNUTLS) && defined(HAVE_GNUTLS)
 # include <gnutls/gnutls.h>
+# include <gnutls/x509.h>
 # define HAVE_TLS
 # undef USE_OPENSSL
 #endif /* USE_GNUTLS and HAVE_GNUTLS */
@@ -131,6 +132,10 @@
 # include <security/pam_misc.h>
 #endif /* HAVE_PAM */
 
+#if defined(WITH_GSASL)
+# include <gsasl.h>
+#endif
+
 #include "gettext.h"
 #ifdef ENABLE_NLS
 # define _(String) gettext(String)
@@ -139,8 +144,8 @@
 # endif /* HAVE_LOCALE_H */
 #else
 # define _(String) (String)
-# define N_(String) String
 #endif /* ENABLE_NLS */
+#define N_(String) String
 
 #ifdef WITH_GUILE
 # include <libguile.h>
@@ -210,6 +215,11 @@
 #define HARD   2
 #define SYNTAX 3
 
+typedef enum anubis_mode {
+	anubis_transparent,
+	anubis_authenticate
+} ANUBIS_MODE;
+
 /* bit values for topt */
 #define T_ERROR             0x00000001
 #define T_SOCKS             0x00000002
@@ -264,14 +274,40 @@
 typedef struct rc_regex RC_REGEX;
 typedef struct assoc ASSOC;
 typedef struct message_struct MESSAGE;
-typedef int (*net_io_t) (void *, char *, size_t, size_t *);
-typedef int (*net_close_t) (void *);
-typedef const char *(*strerror_t) (int);
+
+/* stream.c */
+
+typedef struct net_stream *NET_STREAM;
+typedef int (*stream_io_t) (void *, char *, size_t, size_t *);
+typedef int (*stream_close_t) (void *);
+typedef int (*stream_destroy_t) (void *);
+typedef const char *(*stream_strerror_t) (void *, int);
+
+void stream_create(NET_STREAM *str);
+int stream_set_io(NET_STREAM str,
+		  void *data,
+		  stream_io_t read, stream_io_t write,
+		  stream_close_t close,
+		  stream_destroy_t destroy,
+		  stream_strerror_t strerror);
+int stream_set_read(struct net_stream *str, stream_io_t read);
+int stream_set_write(struct net_stream *str, stream_io_t write);
+int stream_set_strerror(struct net_stream *str, stream_strerror_t strerr);
+int stream_close(NET_STREAM str);
+const char *stream_strerror(NET_STREAM str, int errcode);
+int stream_read(NET_STREAM str, char *buf, size_t size,
+		size_t *nbytes);
+int stream_write(NET_STREAM str, char *buf, size_t size,
+		 size_t *nbytes);
+int stream_readline(NET_STREAM str, char *buf, size_t size,
+		    size_t *nbytes);
+int stream_destroy(NET_STREAM *);
 
 /* main.c */
 void anubis(char *);
 
 /* mem.c */
+extern void (*memory_error)(const char *message);
 void *xmalloc(int);
 void *xrealloc(void *, int);
 char *allocbuf(char *, int);
@@ -290,6 +326,7 @@ void get_options(int, char *[]);
 void get_homedir(char *, char *, int);
 void anubis_getlogin(char *, int);
 void anubis_changeowner(char *);
+int anubis_set_mode(char *modename);
 int  check_superuser(void);
 int  check_username(char *);
 int  check_filemode(char *);
@@ -298,7 +335,6 @@ int  check_filename(char *, time_t *);
 /* errs.c */
 void anubis_error(int, const char *, ...);
 void socket_error(const char *);
-void socks_error(const char *);
 void hostname_error(char *);
 
 /* log.c */
@@ -307,23 +343,23 @@ void info(int, const char *, ...);
 void filelog(char *, char *);
 
 /* net.c */
-void net_set_io(int, net_io_t, net_io_t, net_close_t, strerror_t);
-void net_close(int, void *);
-int  make_remote_connection(char *, unsigned int);
+NET_STREAM make_remote_connection(char *, unsigned int);
 int  bind_and_listen(char *, unsigned int);
-void swrite(int, void *, char *);
-int  recvline(int, void *, void *, int);
-void get_response_smtp(int, void *, char *, int);
-void close_socket(int);
-
-/* proxy.c */
-void check_all_proxies(char *, unsigned int *);
-int  check_socks_proxy(int, char *, unsigned int);
+void swrite(int, NET_STREAM, char *);
+int  recvline(int, NET_STREAM, void *, size_t);
+int  recvline_ptr(int method, NET_STREAM sd, char **vptr, size_t *maxlen);
+void get_response_smtp(int, NET_STREAM, char *, int);
+void close_socket(int sd);
+	
+void net_create_stream(NET_STREAM *str, int fd);
+void net_close_stream(NET_STREAM *sd);
 
 /* daemon.c */
 void daemonize(void);
 void loop(int);
 void stdinout(void);
+void service_unavailable(NET_STREAM *);
+void set_unprivileged_user(void);
 
 /* auth.c */
 void auth_tunnel(void);
@@ -335,6 +371,7 @@ void translate_section_init(void);
 
 /* tunnel.c */
 void smtp_session(void);
+void smtp_session_transparent(void);
 
 /* message.c */
 void message_add_body(MESSAGE *, char *, char *);
@@ -348,28 +385,30 @@ void message_free(MESSAGE *);
 
 /* exec.c */
 char **gen_execargs(const char *);
-int  make_local_connection(char *, char **);
+NET_STREAM make_local_connection(char *, char **);
 char *external_program(int *, char *, char *, char *, int);
 char *exec_argv(int *, char *, char **, char *, char *, int);
 void cleanup_children();
 
 /* esmtp.c */
-void esmtp_auth(void *, char *);
+void esmtp_auth(NET_STREAM, char *);
 
 /* misc.c */
 int anubis_free_list_item(void *item, void *data);
 void assoc_free(ASSOC *);
 ASSOC *header_assoc(char *);
-void destroy_assoc_list(LIST **);
-void destroy_string_list(LIST **);
+void destroy_assoc_list(ANUBIS_LIST **);
+void destroy_string_list(ANUBIS_LIST **);
 void parse_mtaport(char *, char *, unsigned int *);
 void parse_mtahost(char *, char *, unsigned int *);
 void remline(char *, char *);
 void remcrlf(char *);
 char *substitute(char *, char **);
-void change_to_lower(char *);
+void make_lowercase(char *);
+char *get_localname(void);
+char *get_localdomain(void);
 
-/* files.c */
+/* mime.c */
 void message_append_text_file(MESSAGE *, char *);
 void message_append_signature_file(MESSAGE *);
 
@@ -388,6 +427,7 @@ void open_rcfile(int);
 void process_rcfile(int);
 void rcfile_process_section(int, char *, void *, MESSAGE *);
 void rcfile_call_section(int, char *, void *, MESSAGE *);
+char *user_rcfile_name();
 
 /* help.c */
 void print_version(void);
@@ -400,11 +440,17 @@ RETSIGTYPE sig_timeout(int);
 void free_mem(void);
 void quit(int);
 
+/* socks.c */
+#ifdef USE_SOCKS_PROXY
+int check_socks_proxy(int, char *, unsigned int);
+#endif /* USE_SOCKS_PROXY */
+
 /* tls.c or ssl.c */
 #ifdef USE_SSL
 void init_ssl_libs(void);
-void *start_ssl_client(int);
-void *start_ssl_server(int);
+NET_STREAM start_ssl_client(NET_STREAM str, const char *cafile, int verbose);
+NET_STREAM start_ssl_server(NET_STREAM str, const char *cafile,
+			    const char *cert, const char *key, int verbose);
 #endif /* USE_SSL */
 
 /* gpg.c */
@@ -420,9 +466,110 @@ void guile_load_path_append(char *);
 void guile_debug(int);
 void guile_load_program(char *);
 void guile_rewrite_line(char *, const char *);
-void guile_postprocess_proc(char *, LIST **, char **);
+void guile_postprocess_proc(char *, ANUBIS_LIST **, char **);
 void guile_section_init(void);
 #endif /* WITH_GUILE */
 
+/* url.c */
+   
+typedef struct anubis_url {
+	char *method;
+	char *host;
+	char *path;
+	char *user;
+	char *passwd;
+	int argc;
+	ASSOC *argv;
+} ANUBIS_URL;
+
+void anubis_url_destroy(ANUBIS_URL **url);
+int anubis_url_parse(ANUBIS_URL **url, char *str);
+char *anubis_url_full_path(ANUBIS_URL *url);
+const char *anubis_url_get_arg(ANUBIS_URL *url, const char *argname);
+
+/* anubisdb.c */
+
+typedef struct anubis_user {
+  char *smtp_authid;         /* ESMTP authentication ID */
+  char *smtp_passwd;         /* A corresponding password */
+  char *username;            /* System user name to switch to */
+  char *rcfile_name;         /* Optional configuration file. 
+				When NULL, defaults to
+				~username/.anubisrc */
+} ANUBIS_USER;
+
+enum anubis_db_mode {
+	anubis_db_rdonly,
+	anubis_db_rdwr
+};
+
+#define ANUBIS_DB_SUCCESS   0   /* Operations successful */
+#define ANUBIS_DB_FAIL      1   /* Operation failed */
+#define ANUBIS_DB_NOT_FOUND 2   /* Record not found (for db_get_record
+				   only) */
+
+typedef int (*anubis_db_open_t) (void **d, ANUBIS_URL *url,
+				 enum anubis_db_mode mode, char **errp);
+typedef int (*anubis_db_close_t) (void *d);
+typedef int (*anubis_db_io_t) (void *d, const char *key, ANUBIS_USER *rec,
+			       int *ecode);
+typedef const char *(*anubis_db_strerror_t) (void *d, int rc);
+typedef int (*anubis_db_delete_t) (void *d, const char *key, int *ecode);
+typedef int (*anubis_db_get_list_t) (void *d, ANUBIS_LIST *list, int *ecode);
+	     
+int anubis_db_register(const char *dbid, 
+		       anubis_db_open_t _db_open,
+		       anubis_db_close_t _db_close,
+		       anubis_db_io_t _db_get,
+		       anubis_db_io_t _db_put,
+		       anubis_db_delete_t _db_delete,
+		       anubis_db_get_list_t _db_list,
+		       anubis_db_strerror_t _db_strerror);
+int anubis_db_open(char *arg, enum anubis_db_mode mode, void **dptr,
+		   char **errp);
+int anubis_db_close(void **dptr);
+int anubis_db_get_record(void *dptr, const char *key, ANUBIS_USER *rec);
+int anubis_db_put_record(void *dptr, const char *key, ANUBIS_USER *rec);
+int anubis_db_delete_record(void *dptr, const char *key);
+int anubis_db_get_list(void *dptr, ANUBIS_LIST **list);
+const char *anubis_db_strerror(void *dptr);
+void anubis_db_free_record(ANUBIS_USER *rec);
+
+/* dbtext.c */
+void dbtext_init();
+
+/* gdbm.c */
+void gdbm_db_init();
+
+/* mysql.c */
+void mysql_db_init();
+
+/* pgsql.c */
+void pgsql_db_init();
+
+/* transmode.c */
+int anubis_transparent_mode(NET_STREAM *sd_client, struct sockaddr_in *addr);
+
+/* authmode.c */
+int anubis_authenticate_mode(NET_STREAM *sd_client, struct sockaddr_in *addr);
+void anubis_set_password_db(char *arg);
+void asmtp_reply(int code, char *fmt, ...);
+void asmtp_capa_add_prefix(char *prefix, char *name);
+int anubis_get_db_record(const char *username, ANUBIS_USER *usr);
+void authmode_section_init(void);
+
+/* gsasl.c */
+void auth_gsasl_init();
+int anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER *usr, NET_STREAM *stream);
+void anubis_set_mech_list(ANUBIS_LIST *list);
+#if defined(WITH_GSASL)
+void install_gsasl_stream (Gsasl_session_ctx *sess_ctx, NET_STREAM *stream);
+#endif
+
+/* xdatabase.c */
+int xdatabase(char *command);
+void xdatabase_capability(char *reply, size_t reply_size);
+void xdatabase_enable(void);
+
 /* EOF */
 
