@@ -2,7 +2,7 @@
    help.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,12 @@ static char *config_opts[] = {
 #ifdef HAVE_PCRE
 	"PCRE",
 #endif /* HAVE_PCRE */
+#ifdef WITH_GSASL
+	"GSASL",
+#endif /* WITH_GSASL */
+#ifdef WITH_MYSQL
+	"MYSQL",
+#endif /* WITH_MYSQL */
 #ifdef WITH_GUILE
 	"GUILE",
 #endif /* WITH_GUILE */
@@ -50,25 +56,21 @@ static char *config_opts[] = {
 #ifdef USE_LIBWRAP
 	"LIBWRAP",
 #endif /* USE_LIBWRAP */
+#ifdef USE_SOCKS_PROXY
+	"SOCKS",
+#endif /* USE_SOCKS_PROXY */
 #ifdef ENABLE_NLS
 	"NLS",
 #endif /* ENABLE_NLS */
 	NULL
 };
 
-static void
-show_config_options(int delim)
-{
-	int i;
-
-	for (i = 0; config_opts[i]; i++) 
-		printf("%s%c", config_opts[i], delim);
-}
-
 void
 print_config_options(void)
 {
-	show_config_options('\n');
+	int i;
+	for (i = 0; config_opts[i]; i++) 
+		puts(config_opts[i]);
 	exit(0);
 }
 
@@ -76,12 +78,8 @@ void
 print_version(void)
 {
 	printf("\n%s\n", version);
-	printf("%s\n\n", copyright);
-
-	printf(_("Built-in support for: "));
-	show_config_options(' ');
-		
-	puts(_("\n\nGNU Anubis is free software; you can redistribute it and/or modify\n"
+	puts(copyright);
+	puts(_("\nGNU Anubis is free software; you can redistribute it and/or modify\n"
 	       "it under the terms of the GNU General Public License as published by\n"
 	       "the Free Software Foundation; either version 2 of the License, or\n"
 	       "(at your option) any later version."));
@@ -109,6 +107,8 @@ print_usage(void)
 	puts(_(" -l, --local-mta FILE         Execute a local SMTP server, which works on\n"
 	       "                              standard input and output (inetd-type program).\n"
 	       "                              This option excludes the '--remote-mta' option."));
+	puts(_(" -m, --mode=MODE              Select operation mode."));
+	puts(_("                              MODE is either \"transparent\" or \"auth\"")); 
 	puts(_(" -f, --foreground             Foreground mode."));
 	puts(_(" -i, --stdio                  Use the SMTP protocol (OMP/Tunnel) as described\n"
 	       "                              in RFC 821 on standard input and output."));
