@@ -1,6 +1,6 @@
 /*
-   GNU Anubis v3.9.93 -- an outgoing mail processor and the SMTP tunnel.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   GNU Anubis v3.9.94 -- an outgoing mail processor and the SMTP tunnel.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,8 +23,10 @@
 #include "headers.h"
 #include "extern.h"
 
+ANUBIS_MODE anubis_mode = anubis_transparent;
+
 const char version[] = "GNU Anubis v"VERSION;
-const char copyright[] = "Copyright (C) 2001, 2002, 2003 The Anubis Team.";
+const char copyright[] = "Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.";
 
 struct options_struct options;
 struct session_struct session;
@@ -33,8 +35,10 @@ struct secure_struct secure;
 #endif /* HAVE_TLS or HAVE_SSL */
 
 unsigned long topt;
-void *remote_client;
-void *remote_server;
+NET_STREAM remote_client;
+NET_STREAM remote_server;
+
+char *anubis_domain;   /* Local domain for EHLO in authentication mode */
 
 #ifdef WITH_GUILE
 void
@@ -47,13 +51,22 @@ anubis_core(void)
 # define anubis_core() anubis(NULL)
 #endif /* WITH_GUILE */
 
+static void
+anubis_memory_error(const char *msg)
+{
+	anubis_error(HARD, msg);
+	quit(EXIT_FAILURE); /* force exit */
+}
+
 int
 main(int argc, char *argv[])
 {
+	/* Register memory error printer */
+	memory_error = anubis_memory_error;
+	
 	/*
 	   Signal handling.
 	*/
-
 	signal(SIGILL,  sig_exit);
 	signal(SIGINT,  sig_exit);
 	signal(SIGTERM, sig_exit);
@@ -77,10 +90,12 @@ main(int argc, char *argv[])
 
 	options.termlevel = NORMAL;
 	options.uloglevel = FAILS;
-	session.tunnel_port = 24; /* private mail system */
+	session.anubis_port = 24; /* private mail system */
 	session.mta_port = 25;
-	session.socks_port = 1080;
 
+#ifdef USE_SOCKS_PROXY
+	session.socks_port = 1080;
+#endif
 	/*
 	   Process the command line options.
 	*/
@@ -89,6 +104,23 @@ main(int argc, char *argv[])
 	get_options(argc, argv);
 	anubis_getlogin(session.supervisor, sizeof(session.supervisor));
 
+	/*
+	   Initialize various database formats
+	*/
+
+#ifdef WITH_GSASL
+	dbtext_init();
+# ifdef HAVE_LIBGDBM
+	gdbm_db_init();
+# endif
+# ifdef WITH_MYSQL
+	mysql_db_init();
+# endif
+# ifdef WITH_PGSQL
+	pgsql_db_init();
+# endif
+#endif /* WITH_GSASL */
+	
 	/*
 	   Initialize the rc parsing subsystem.
 	   Read the system configuration file (SUPERVISOR).
@@ -144,7 +176,7 @@ anubis(char *arg)
 	}
 	else { /* daemon */
 		int sd_bind;
-		sd_bind = bind_and_listen(session.tunnel, session.tunnel_port);
+		sd_bind = bind_and_listen(session.anubis, session.anubis_port);
 		if (topt & T_FOREGROUND_INIT)
 			topt |= T_FOREGROUND;
 		else
