@@ -33,7 +33,7 @@
 char *
 parse_line_option(char *ptr)
 {
-	while (isspace(*ptr))
+	while (isspace(*(u_char*)ptr))
 		ptr++;
 
 	remcrlf(ptr);
@@ -88,7 +88,7 @@ struct rc_kwdef translate_kw[] = {
 
 
 int
-translate_parser(int method, int key, LIST *arglist, void *inv_data,
+translate_parser(int method, int key, ANUBIS_LIST *arglist, void *inv_data,
 		 void *func_data, MESSAGE *msg)
 {
 	struct translate_env *env = func_data;
@@ -208,11 +208,6 @@ static struct rc_secdef_child translate_secdef_child = {
 	NULL
 };
 
-struct rc_secdef translate_section = {
-	"TRANSLATE",
-	&translate_secdef_child
-};
-
 void
 translate_section_init(void)
 {
