@@ -37,7 +37,7 @@ anubis_free_list_item(void *item, void *data)
 /* String lists */
 
 void
-destroy_string_list(LIST **plist)
+destroy_string_list(ANUBIS_LIST **plist)
 {
 	list_destroy(plist, anubis_free_list_item, NULL);
 }
@@ -50,7 +50,7 @@ _assoc_free(void *item, void *data)
 }
 
 void
-destroy_assoc_list(LIST **plist)
+destroy_assoc_list(ANUBIS_LIST **plist)
 {
 	list_destroy(plist, _assoc_free, NULL);
 }
@@ -102,11 +102,11 @@ assoc_to_header(ASSOC *asc)
 *****************************/
 
 void
-parse_mtaport(char *opt, char *host, unsigned int *port)
+parse_mtaport (char *opt, char *host, unsigned int *port)
 {
 	struct servent *p;
 	char opt_tmp[256];
-	char *port_tmp = 0;
+	char *port_tmp = NULL;
 	safe_strcpy(opt_tmp, opt);
 
 	if ((port_tmp = strrchr(opt_tmp, ':'))) {
@@ -117,7 +117,6 @@ parse_mtaport(char *opt, char *host, unsigned int *port)
 			*port = (unsigned int)atoi(port_tmp);
 	}
 	strncpy(host, opt_tmp, sizeof(session.mta) - 1);
-	return;
 }
 
 /**************************
@@ -125,11 +124,11 @@ parse_mtaport(char *opt, char *host, unsigned int *port)
 ***************************/
 
 void
-parse_mtahost(char *opt, char *host, unsigned int *port)
+parse_mtahost (char *opt, char *host, unsigned int *port)
 {
 	struct servent *p;
 	char opt_tmp[256];
-	char *port_tmp = 0;
+	char *port_tmp = NULL;
 	safe_strcpy(opt_tmp, opt);
 
 	if ((port_tmp = strrchr(opt_tmp, ':'))) {
@@ -138,7 +137,7 @@ parse_mtahost(char *opt, char *host, unsigned int *port)
 			*port = ntohs(p->s_port);
 		else
 			*port = (unsigned int)atoi(port_tmp);
-		strncpy(host, opt_tmp, sizeof(session.tunnel) - 1);
+		strncpy(host, opt_tmp, sizeof(session.anubis) - 1);
 	}
 	else { /* only port number available */
 		if ((p = getservbyname(opt, "tcp")))
@@ -146,7 +145,6 @@ parse_mtahost(char *opt, char *host, unsigned int *port)
 		else
 			*port = (unsigned int)atoi(opt);
 	}
-	return;
 }
 
 /*********************
@@ -154,16 +152,16 @@ parse_mtahost(char *opt, char *host, unsigned int *port)
 **********************/
 
 void
-remline(char *s, char *line)
+remline (char *s, char *line)
 {
-	char *pos1 = 0;
-	char *pos2 = 0;
+	char *pos1 = NULL;
+	char *pos2 = NULL;
 	int len;
 
-	if (s == 0 || line == 0)
+	if (!s || !line)
 		return;
 	pos1 = strstr(s, line);
-	if (pos1 == 0)
+	if (!pos1)
 		return;
 
 	pos2 = pos1;
@@ -174,15 +172,14 @@ remline(char *s, char *line)
 	len = strlen(pos2);
 	pos2 = (char *)memmove(pos1, pos2, len);
 	pos2[len] = '\0';
-	return;
 }
 
 void
-remcrlf(char *s)
+remcrlf (char *s)
 {
 	int len;
 
-	if (s == 0)
+	if (!s)
 		return;
 	len = strlen(s);
 
@@ -196,7 +193,6 @@ remcrlf(char *s)
 		s[len - 1] = '\0';
 	else if (len >= 1 && s[len - 1] == '\n') /* LF */
 		s[len - 1] = '\0';
-	return;
 }
 
 /***********************************
@@ -204,20 +200,20 @@ remcrlf(char *s)
 ************************************/
 
 static char *
-insert(char *inbuf, char *sign, char *fill_in)
+insert (char *inbuf, char *sign, char *fill_in)
 {
 	int len1 = 0;
 	int len2 = 0;
 	int psign_len = 0;
-	char *psign = 0;
-	char *outbuf = 0;
+	char *psign = NULL;
+	char *outbuf = NULL;
 
-	if (inbuf == 0 || sign == 0 || fill_in == 0)
-		return 0;
+	if (!inbuf || !sign || !fill_in)
+		return NULL;
 
 	psign = strstr(inbuf, sign);
-	if (psign == 0)
-		return 0;
+	if (!psign)
+		return NULL;
 
 	psign_len = strlen(psign);
 	len1 = strlen(inbuf);
@@ -235,8 +231,7 @@ insert(char *inbuf, char *sign, char *fill_in)
 	strcat(outbuf, psign);
 
 	if (strstr(outbuf, sign)) {
-		char *outbuf2 = 0;
-		outbuf2 = insert(outbuf, sign, fill_in);
+		char *outbuf2 = insert(outbuf, sign, fill_in);
 		free(outbuf);
 		outbuf = outbuf2;
 	}
@@ -245,16 +240,16 @@ insert(char *inbuf, char *sign, char *fill_in)
 }
 
 char *
-substitute(char *inbuf, char **subbuf)
+substitute (char *inbuf, char **subbuf)
 {
 	char **tmp = subbuf;
-	char *tmpout = 0;
-	char *tmpbuf = 0;
+	char *tmpout = NULL;
+	char *tmpbuf = NULL;
 	char sign[5];
 	int i = 0;
 
-	if (inbuf == 0 || subbuf == 0)
-		return 0;
+	if (!inbuf || !subbuf)
+		return NULL;
 
 	tmpbuf = allocbuf(inbuf, 0);
 	tmp++;
@@ -273,23 +268,75 @@ substitute(char *inbuf, char **subbuf)
 	return tmpbuf;
 }
 
-/***************************
- Change to lower characters
-****************************/
+/********************
+ Change to lowercase
+*********************/
 
 void
-change_to_lower(char *s)
+make_lowercase (char *s)
 {
-	int c;
-	int len;
+	int c, len;
 
-	if (s == 0)
+	if (!s)
 		return;
 	len = strlen(s);
 
 	for (c = len - 1; c >= 0; c--)
 		s[c] = tolower((unsigned char)s[c]);
-	return;
+}
+
+char *
+get_localname (void)
+{
+	static char *localname = NULL;
+
+	if (!localname)	{
+		char *name;
+		int name_len = 256;
+		int status = 1;
+		struct hostent *hp;
+
+		name = malloc (name_len);
+		while (name
+		       && (status = gethostname (name, name_len)) == 0
+		       && !memchr (name, 0, name_len)) {
+			name_len *= 2;
+			name = realloc (name, name_len);
+		}
+		if (status) {
+			info (NORMAL,
+			      _("Can't find out my own hostname"));
+			exit (1);
+		}
+
+		hp = gethostbyname (name);
+		if (hp)	{
+			struct in_addr inaddr;
+			inaddr.s_addr = *(unsigned int *) hp->h_addr;
+			hp = gethostbyaddr ((const char *) &inaddr,
+					    sizeof (struct in_addr), AF_INET);
+			if (hp) {
+				free (name);
+				name = strdup ((char *) hp->h_name);
+			}
+		}
+		localname = name;
+	}
+	return localname;
+}
+
+char *
+get_localdomain (void)
+{
+	if (!anubis_domain) {
+		char *localname = get_localname(),
+		     *p = strchr(localname, '.');
+		if (!p)
+			anubis_domain = strdup(localname);
+		else 
+			anubis_domain = strdup(p+1);
+	}
+	return anubis_domain;
 }
 
 /* EOF */
