@@ -55,7 +55,9 @@ enum rc_stmt_type {          /* Statement type: */
 
 struct rc_asgn {             /* Assignment */
 	char *lhs;           /* Left-hand side: A keyword */
-	LIST *rhs;    /* Right-hand side: A list of character strings */
+	ANUBIS_LIST *rhs;    /* Right-hand side: A list of character strings */
+	int flags;           /* Flags control various aspects of assignment
+				functionality */
 };
 
 enum bool_op {               /* Boolean operator */
@@ -134,12 +136,16 @@ struct rc_stmt {             /* General statement representation */
 #define RC_KW_UNKNOWN 1
 #define RC_KW_ERROR   2
 
-typedef int (*rc_kw_parser_t)(int method, int key, LIST *arg,
+typedef int (*rc_kw_parser_t)(int method, int key, ANUBIS_LIST *arg,
 			      void *inv_data, void *func_data, MESSAGE *msg);
 
+/* Keyword flags */
+#define KWF_HIDDEN 0x0001   /* Replace RHS with stars in debugging output */
+
 struct rc_kwdef {
-	char *name;
-	int tok;
+	char *name;         /* Keyword name */
+	int tok;            /* Assigned token number */
+	int flags;          /* Flags controlling debugging output, etc. */
 };
 
 struct rc_secdef_child {
@@ -166,6 +172,10 @@ struct rc_secdef {
 	struct rc_secdef_child *child;
 };
 	
+typedef void (*RC_ERROR_PRINTER) (void *data, 
+				  const char *filename, int line,
+				  const char *fmt, va_list ap);
+
 /* Global data */
 extern int cfg_line_num;
 extern char *cfg_file; 
@@ -180,6 +190,7 @@ RC_SECTION *rc_section_lookup(RC_SECTION *, char *);
 void rc_section_link(RC_SECTION **, RC_SECTION *);
 void rc_secdef_add_child(struct rc_secdef *, struct rc_secdef_child *);
 RC_SECTION *rc_parse(char *);
+RC_SECTION *rc_parse_ep(char *name, RC_ERROR_PRINTER errprn, void *data);
 void rc_section_list_destroy(RC_SECTION **);
 int rc_run_cond(char *, int, char *);
 void rc_run_section_list(int, RC_SECTION *, struct rc_secdef *);
