@@ -2,7 +2,7 @@
    tunnel.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 
 static int  transfer_command(MESSAGE *, char *);
 static int  process_command(MESSAGE *, char *);
-static void transfer_header(LIST *);
+static void transfer_header(ANUBIS_LIST *);
 static void transfer_body(MESSAGE *);
 static void process_data(MESSAGE *);
 
@@ -43,7 +43,7 @@ static void process_data(MESSAGE *);
    snd sent in multiline lines. */
    
 static void
-get_boundary(MESSAGE *msg, char *line)
+get_boundary (MESSAGE *msg, char *line)
 {
 	char boundary_buf[LINEBUFFER+1], *p;
 
@@ -52,7 +52,7 @@ get_boundary(MESSAGE *msg, char *line)
 
 	/* Downcase the string to help search for boundary */
 	safe_strcpy(boundary_buf, line);
-	change_to_lower(boundary_buf);
+	make_lowercase(boundary_buf);
 	p = strstr(boundary_buf, "boundary=");
 	if (!p)
 		return;
@@ -70,7 +70,7 @@ get_boundary(MESSAGE *msg, char *line)
 }
 
 static void
-add_header(LIST *list, char *line)
+add_header (ANUBIS_LIST *list, char *line)
 {
 	ASSOC *asc = header_assoc(line);
 	list_append(list, asc);
@@ -90,7 +90,7 @@ add_header(LIST *list, char *line)
 }
 
 static void
-collect_headers(MESSAGE *msg)
+collect_headers (MESSAGE *msg)
 {
 	char buf[LINEBUFFER+1];
 	char *line = NULL;
@@ -122,7 +122,7 @@ collect_headers(MESSAGE *msg)
 }
 
 static void
-write_header_line(void *sd_server, char *line)
+write_header_line (NET_STREAM sd_server, char *line)
 {
 	char *p;
 
@@ -134,7 +134,7 @@ write_header_line(void *sd_server, char *line)
 }
 			
 static void
-write_assoc(void *sd_server, ASSOC *entry)
+write_assoc (NET_STREAM sd_server, ASSOC *entry)
 {
 	if (entry->key) {
 		if (strcmp(entry->key, X_ANUBIS_RULE_HEADER) == 0)
@@ -146,7 +146,7 @@ write_assoc(void *sd_server, ASSOC *entry)
 }
 			
 void
-send_header(void *sd_server, LIST *list)
+send_header (NET_STREAM sd_server, ANUBIS_LIST *list)
 {
 	ASSOC *p;
 	ITERATOR *itr = iterator_create(list);
@@ -157,7 +157,7 @@ send_header(void *sd_server, LIST *list)
 }
 
 void
-send_string_list(void *sd_server, LIST *list)
+send_string_list (NET_STREAM sd_server, ANUBIS_LIST *list)
 {
 	char *p;
 	ITERATOR *itr = iterator_create(list);
@@ -185,7 +185,7 @@ send_string_list(void *sd_server, LIST *list)
 #define ST_DONE  3
 
 static void
-collect_body(MESSAGE *msg)
+collect_body (MESSAGE *msg)
 {
 	int nread;
 	char buf[LINEBUFFER+1];
@@ -241,7 +241,7 @@ collect_body(MESSAGE *msg)
 }
 
 void
-send_body(MESSAGE *msg, void *sd_server)
+send_body (MESSAGE *msg, NET_STREAM sd_server)
 {
 	char *p;
 
@@ -276,7 +276,7 @@ send_body(MESSAGE *msg, void *sd_server)
 *******************/
 
 void
-smtp_session()
+smtp_session_transparent (void)
 {
 	char command[LINEBUFFER+1];
 	MESSAGE msg;
@@ -332,24 +332,72 @@ smtp_session()
 	return;
 }
 
+void
+smtp_begin (void)
+{
+	char command[LINEBUFFER+1];
+
+	get_response_smtp(CLIENT, remote_server, command, sizeof(command) - 1);
+}
+
+void
+smtp_session (void)
+{
+	char command[LINEBUFFER+1];
+	MESSAGE msg;
+
+	info(VERBOSE, _("Starting SMTP session..."));
+	smtp_begin ();
+	info(VERBOSE, _("Transferring message(s)..."));
+
+	message_init(&msg);
+	while (recvline(SERVER, remote_client, command, sizeof(command) - 1)) {
+		if (process_command(&msg, command))
+			continue;
+
+		if (topt & T_ERROR)
+			break;
+		
+		if (transfer_command(&msg, command) == 0)
+			break;
+	}
+
+	message_free(&msg);
+	return;
+}
+
 /********************
   THE MAIL COMMANDS
 *********************/
 
 static void
-save_command(MESSAGE *msg, char *line)
+save_command (MESSAGE *msg, char *line)
 {
 	int i;
 	ASSOC *asc = xmalloc(sizeof(*asc));
 
-	for (i = 0; line[i] && isspace((u_char)line[i]); i++)
+	for (i = 0; line[i] && !isspace((u_char)line[i]); i++)
 		;
 
+	if (i == 4) {
+		char *expect = NULL;
+		if (memcmp(line, "mail", 4) == 0)
+			expect = " from:";
+		else if (memcmp(line, "rcpt", 4) == 0)
+			expect = " to:";
+		if (expect) {
+			int n = strlen(expect);
+			if (strncmp(&line[i], expect, n) == 0)
+				i += n;
+		}
+	}
+			
 	asc->key = xmalloc(i + 1);
 	memcpy(asc->key, line, i);
 	asc->key[i] = 0;
 	for (; line[i] && isspace((u_char)line[i]); i++)
 		;
+
 	if (line[i])
 		asc->value = strdup(&line[i]);
 	else
@@ -358,7 +406,7 @@ save_command(MESSAGE *msg, char *line)
 }
 
 static int
-handle_starttls(char *command)
+handle_starttls (char *command)
 {
 #ifdef USE_SSL
 	if (topt & T_SSL_FINISHED) {
@@ -395,7 +443,9 @@ handle_starttls(char *command)
 			return 0;
 		}
 
-		secure.client = start_ssl_client((int)remote_server);
+		secure.client = start_ssl_client(remote_server,
+						 secure.cafile,
+						 options.termlevel > NORMAL);
 		if (!secure.client || (topt & T_ERROR))
 			return 0;
 		remote_server = (void *)secure.client;
@@ -424,13 +474,17 @@ handle_starttls(char *command)
 		check_filemode(secure.key);
 
 	swrite(SERVER, remote_client, "220 2.0.0 Ready to start TLS"CRLF);
-	secure.server = start_ssl_server((int)remote_client);
+	secure.server = start_ssl_server(remote_client,
+					 secure.cafile,
+					 secure.cert,
+					 secure.key,
+					 options.termlevel > NORMAL);
 	if (!secure.server || (topt & T_ERROR)) {
 		swrite(SERVER, remote_client,
 		       "454 4.3.3 TLS not available"CRLF);
 		return 0;
 	}
-	remote_client = (void *)secure.server;
+	remote_client = secure.server;
 	topt |= T_SSL_FINISHED;
 #else
 	swrite(SERVER, remote_client, "503 5.5.0 TLS not available"CRLF);
@@ -440,28 +494,33 @@ handle_starttls(char *command)
 }
 
 static int
-process_command(MESSAGE *msg, char *command)
+process_command (MESSAGE *msg, char *command)
 {
 	char buf[LINEBUFFER+1];
 	
 	safe_strcpy(buf, command); /* make a back-up */
+	remcrlf(buf);
+	make_lowercase(buf);
 	save_command(msg, buf);
-
-	change_to_lower(buf);
 	
 	if (strncmp(buf, "starttls", 8) == 0) 
 		return handle_starttls(command);
+	else if (strncmp(buf, "xdatabase", 8) == 0) 
+		return xdatabase (buf);
 	return 0;
 }
 
 static int
-handle_ehlo(char *command, char *reply, size_t reply_size)
+handle_ehlo (char *command, char *reply, size_t reply_size)
 {
 	get_response_smtp(CLIENT, remote_server, reply, reply_size - 1);
 
 	if (strstr(reply, "STARTTLS"))
-		topt |= T_STARTTLS; /* Yes, we can use the TLS/SSL encryption. */
+		topt |= T_STARTTLS; /* Yes, we can use the TLS/SSL
+				       encryption. */
 
+	xdatabase_capability (reply, reply_size);
+	
 #ifdef USE_SSL
 	if ((topt & T_SSL_ONEWAY)
 	    && (topt & T_STARTTLS)
@@ -493,8 +552,10 @@ handle_ehlo(char *command, char *reply, size_t reply_size)
 			swrite(SERVER, remote_client, reply);
 			return 1;
 		}
-
-		secure.client = start_ssl_client((int)remote_server);
+		
+		secure.client = start_ssl_client(remote_server,
+						 secure.cafile,
+						 options.termlevel > NORMAL);
 		if (!secure.client || (topt & T_ERROR)) {
 			topt &= ~T_ERROR;
 			topt &= ~T_SSL_ONEWAY;
@@ -557,20 +618,20 @@ handle_ehlo(char *command, char *reply, size_t reply_size)
 }
 
 static int
-transfer_command(MESSAGE *msg, char *command)
+transfer_command (MESSAGE *msg, char *command)
 {
 	char reply[2 * LINEBUFFER+1];
 	char buf[LINEBUFFER+1];
 
 	safe_strcpy(buf, command);
-	change_to_lower(buf);
+	make_lowercase(buf);
 	swrite(CLIENT, remote_server, command);
 	if (topt & T_ERROR)
 		return 0;
 
 	if (strncmp(buf, "ehlo", 4) == 0) {
 		if (handle_ehlo(command, reply, sizeof reply))
-			return 1;
+			return 0;
 	} else
 		get_response_smtp(CLIENT, remote_server, reply, sizeof reply);
 
@@ -596,7 +657,7 @@ transfer_command(MESSAGE *msg, char *command)
 }
 
 void
-process_data(MESSAGE *msg)
+process_data (MESSAGE *msg)
 {
 	char buf[LINEBUFFER+1];
 
@@ -619,19 +680,18 @@ process_data(MESSAGE *msg)
 }
 	
 static void
-transfer_header(LIST *header_buf)
+transfer_header (ANUBIS_LIST *header_buf)
 {
 	send_header(remote_server, header_buf);
 	swrite(CLIENT, remote_server, CRLF);
 }
 
-
 /***************
   MESSAGE BODY
 ****************/
 
 static void
-raw_transfer()
+raw_transfer (void)
 {
 	int nread;
 	char buf[LINEBUFFER+1];
@@ -645,7 +705,7 @@ raw_transfer()
 }
 
 void
-transfer_body(MESSAGE *msg)
+transfer_body (MESSAGE *msg)
 {
 	if (msg->boundary) {
 		send_body(msg, remote_server);
@@ -657,6 +717,5 @@ transfer_body(MESSAGE *msg)
 	swrite(CLIENT, remote_server, "."CRLF);
 }
 
-
 /* EOF */
 
